unit Polygon;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt, Sprite;

type

  TRotation = ( rotNone, rotClockwise, rotCounterClockwise );

  TVertex = class( TObject )
  private
     ptPoint: TPoint;
     nCol: byte;
  protected
  public
     constructor Create( pt: TPoint; nColor: byte );
  end;

  TPolygonSprite = class( TSprite )
  private
  protected
     lstVertices: TList;
     lstPreXformed: TList;
     procedure AddVertices( const nCol: byte ); virtual; abstract;
     procedure Render( winGPnt: TWinGPaintBox ); override;
     procedure Rotate( nAngle: integer );
  public
     Spin: TRotation;
     SpinSpeed: integer;
     Angle: integer;
     constructor CreateColor( const nCol: byte );
     destructor Destroy; override;
  end;

  TSquareSprite = class( TPolygonSprite )
  private
  protected
     procedure AddVertices( const nCol: byte ); override;
  public
  end;

var
  k: integer;
  arSin: array[0..360] of single;
  arCos: array[0..360] of single;

implementation

constructor TVertex.Create( pt: TPoint; nColor: byte );
begin
  ptPoint := pt;
  nCol := nColor;
end;

constructor TPolygonSprite.CreateColor( const nCol: byte );
var
  i: integer;
  vert: TVertex;
begin
  inherited Create;
  Spin := rotNone;
  SpinSpeed := 10;
  Angle := 0;
  lstVertices := TList.Create;
  lstPreXformed := TList.Create;
  AddVertices( nCol );
  for i := 0 to lstVertices.Count - 1 do
     begin
        vert := TVertex( lstVertices[i] );
        lstPreXformed.Add( TVertex.Create( vert.ptPoint, vert.nCol ) );
     end;
end;

destructor TPolygonSprite.Destroy;
var
  i: integer;
begin
  for i := 0 to lstVertices.Count - 1 do
     begin
        TVertex( lstVertices[i] ).Free;
        TVertex( lstPreXformed[i] ).Free;
     end;
  lstVertices.Clear;
  lstVertices.Free;
  lstPreXformed.Clear;
  lstPreXformed.Free;
  Spin := rotNone;
  inherited Destroy;
end;

(*********************************************
Spinning of the polygon is not considered important,
so it is done within render so it will not take
up processing time if the sprite is off the edge
of the surface.
*********************************************)
procedure TPolygonSprite.Render( winGPnt: TWinGPaintBox );
var
  i: integer;
  ptFrom, ptTo: TPoint;

  procedure AdjustPoints;
  begin
     Inc( ptFrom.X, ptPosition.X );
     Inc( ptFrom.Y, ptPosition.Y );
     Inc( ptTo.X, ptPosition.X );
     Inc( ptTo.Y, ptPosition.Y );
  end;

begin
  case Spin of
     rotClockwise:
        begin
           Inc( Angle, SpinSpeed );
           if Angle > 360 then
              Angle := Angle - 360;
        end;
     rotCounterClockwise:
        begin
           Dec( Angle, SpinSpeed );
           if Angle < 0 then
              Angle := Angle + 360;
        end;
  end;

  if Angle <> 0 then
     Rotate( Angle );

  for i := 1 to lstVertices.Count - 1 do
     begin
        ptFrom := TVertex( lstVertices[i - 1] ).ptPoint;
        ptTo := TVertex( lstVertices[i] ).ptPoint;
        AdjustPoints;
        winGPnt.Line( ptFrom, ptTo, TVertex( lstVertices[i - 1] ).nCol );
     end;

  ptFrom := TVertex( lstVertices[lstVertices.Count - 1] ).ptPoint;
  ptTo := TVertex( lstVertices[0] ).ptPoint;
  AdjustPoints;

  WinGPnt.Line( ptFrom, ptTo, TVertex( lstVertices[i - 1] ).nCol );
end;

procedure TPolygonSprite.Rotate( nAngle: integer );
var
  xSin, xCos: single;
  i: integer;
  vert, vertPre: TVertex;
begin
  if nAngle >= 0 then
     begin
        xSin := arSin[nAngle];
        xCos := arCos[nAngle];
     end
  else
     begin
        xSin := arSin[nAngle + 360];
        xCos := arCos[nAngle + 360];
     end;
  for i := 0 to lstVertices.Count - 1 do
     begin
        vert := TVertex( lstVertices[i] );
        vertPre := TVertex( lstPreXFormed[i] );
        begin
           vert.ptPoint.X := Round( vertPre.ptPoint.X * xCos - vertPre.ptPoint.Y * xSin );
           vert.ptPoint.Y := Round( vertPre.ptPoint.Y * xCos + vertPre.ptPoint.X * xSin );
        end;
     end;
end;

(*********************************************
TSquareSprite
An example descendant of TPolygonSprite.
*********************************************)
procedure TSquareSprite.AddVertices( const nCol: byte );
begin
  lstVertices.Add( TVertex.Create( Point( -5, -5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( 5, -5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( 5, 5 ), nCol ) );
  lstVertices.Add( TVertex.Create( Point( -5, 5 ), nCol ) );
  Spin := rotClockwise;
end;

(*********************************************
Init code creates a table of sine and cosine values.
*********************************************)
initialization

for k := 0 to 360 do
  begin
     arCos[k] := Cos( k * 3.14159 / 180.0 );
     arSin[k] := Sin( k * 3.14159 / 180.0 );
  end;

end.
